const path = require('path');
const { fork } = require('child_process');

const wait = 5 * 1000;
const script = path.resolve(__dirname, '..', 'backend', 'service.js');
const cwd = path.resolve(__dirname, '..');

let shouldQuit = false;
let child;

function startChild() {
  if (shouldQuit) {
    process.exit();
  }

  console.log('Starting ...');

  child = fork(script, {
    cwd: cwd,
    stdio: 'inherit',
    env: {
      NODE_TLS_REJECT_UNAUTHORIZED: 0,
    },
  });

  // When the child dies, attempt to restart based on configuration
  child.on('exit', () => {
    if (shouldQuit) {
      process.exit();
    }

    child = null;

    setTimeout(startChild, wait);
  });
}

function handleKillSignal() {
  shouldQuit = true;

  if (child) {
    child.send('shutdown');
  } else {
    process.exit();
  }
}

process.on('exit', handleKillSignal);
process.on('SIGINT', handleKillSignal);
process.on('SIGTERM', handleKillSignal);

startChild();